#include "uiparser.h"
#include <stdio.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Remove white spaces, tabs, \r and \n from content.
 * PRE          :  Content to be cleaned.
 */
void
cleanContent(char * content);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  If path and the path of node match return 1. Otherwise 0.
 * PRE          :  node is the node to consider, and path is the path searched.
 */
int
match(xmlNode * node, 
      char * path);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans calls pc to phone window branch, and populates structure Pc2phoneWindow.
 * PRE          :
 */
void
populatePc2phoneWindow(xmlNode * pc2phoneWindowNode, 
                       struct Pc2phoneWindow * pc2phoneWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans preference window branch, and populates structure PrefsWindow.
 * PRE          :
 */
void
populatePrefsWindow(xmlNode * prefsWindowNode, 
                    struct PrefsWindow * prefsWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans calls history window branch, and populates structure CallsHistoryWindow.
 * PRE          :
 */
void
populateCallsHistoryWindow(xmlNode * CallsHistoryWindowNode, 
                    struct CallsHistoryWindow * callsHistoryWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans history window branch, and populates structure HistoryWindow.
 * PRE          :
 */
void
populateHistoryWindow(xmlNode * historyWindowNode, 
                      struct HistoryWindow * historyWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans chat window branch, and populates structure ChatWindow.
 * PRE          :
 */
void
populateChatWindow(xmlNode * chatWindowNode, 
                   struct ChatWindow * chatWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans address book window branch, and populates structure AddressBookWindow.
 * PRE          :
 */
void
populateAddressBookWindow(xmlNode * addressBookWindowNode, 
                          struct AddressBookWindow * addressBookWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans druid window branch, and populates structure DruidWindow.
 * PRE          :
 */
void
populateDruidWindow(xmlNode * DruidWindowNode, 
                    struct DruidWindow * druidWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans accounts window branch, and populates structure AccountsWindow.
 * PRE          :
 */
void
populateAccountsWindow(xmlNode * accountsWindowNode, 
                       struct AccountsWindow * accountsWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans main window branch, and populates structure MainWindow.
 * PRE          :
 */
void
populateMainWindow(xmlNode * mainWindowNode, 
                   struct MainWindow * mainWindow);


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans xml tree, and populates structure userInterface.
 * PRE          :  node is the root node to consider, and userInterface is where
            node's values are stored.
 */
void
populateUserInterfaceStruct(xmlNode * node, 
                            struct UI * userInterface);



/* DESCRIPTION  :  /
 * BEHAVIOR     :  Remove white spaces, tabs, \r and \n from content.
 * PRE          :  Content to be cleaned.
 */
void
cleanContent(xmlChar * content, char * cleanContent)
{
  char * position = cleanContent;
  char * begin =    (char *)content;
  char * end  =     (char *)content + strlen((char *)content);

  /* Find last valid character */
  while(*end == ' '  ||
        *end == '\t' ||
        *end == '\n' ||
        *end == '\r' ||
        *end == '\0')
  {
    end--;

    /* In case there is no valid character */
    if(begin >= end)
    {
      strcpy(cleanContent, "");

      return;
    }
  }

  /* Find first valied character */
  while(*begin == ' '  ||
        *begin == '\t' ||
        *begin == '\n' ||
        *begin == '\r')
  {
    begin++;
  }

  /* Copy valid characters from the beginning*/
  while(begin <= end)
  {
    *position++ = *begin++;
  }

  *position = '\0';
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  If path and the path of node match return 1. Otherwise 0.
 * PRE          :  node is the node to consider, and path is the path searched.
 */
int
match(xmlNode * node, 
      char * path)
{
  if(strcmp((char *) xmlGetNodePath(node), path) == 0)
  {
    return 1;
  }

  return 0;
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans calls pc to phone window branch, and populates structure Pc2phoneWindow.
 * PRE          :
 */
void
populatePc2phoneWindow(xmlNode * pc2phoneWindowNode, 
                       struct Pc2phoneWindow * pc2phoneWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = pc2phoneWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/phoneWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), pc2phoneWindow->title.text);
      }
    }

    populatePc2phoneWindow(cur_node->children, pc2phoneWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans preference window branch, and populates structure PrefsWindow.
 * PRE          :
 */
void
populatePrefsWindow(xmlNode * prefsWindowNode, 
                    struct PrefsWindow * prefsWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = prefsWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/prefsWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), prefsWindow->title.text);
      }
    }

    populatePrefsWindow(cur_node->children, prefsWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans calls history window branch, and populates structure CallsHistoryWindow.
 * PRE          :
 */
void
populateCallsHistoryWindow(xmlNode * callsHistoryWindowNode, 
                    struct CallsHistoryWindow * callsHistoryWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = callsHistoryWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/callsHistoryWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), callsHistoryWindow->title.text);
      }
    }

    populateCallsHistoryWindow(cur_node->children, callsHistoryWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans history window branch, and populates structure HistoryWindow.
 * PRE          :
 */
void
populateHistoryWindow(xmlNode * historyWindowNode, 
                      struct HistoryWindow * historyWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = historyWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/historyWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), historyWindow->title.text);
      }
    }

    populateHistoryWindow(cur_node->children, historyWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans chat window branch, and populates structure ChatWindow.
 * PRE          :
 */
void
populateChatWindow(xmlNode * chatWindowNode, 
                   struct ChatWindow * chatWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = chatWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/chatWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), chatWindow->title.text);
      }
    }

    populateChatWindow(cur_node->children, chatWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans address book window branch, and populates structure AddressBookWindow.
 * PRE          :
 */
void
populateAddressBookWindow(xmlNode * addressBookWindowNode, 
                          struct AddressBookWindow * addressBookWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = addressBookWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/addressBookWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), addressBookWindow->title.text);
      }
    }

    populateAddressBookWindow(cur_node->children, addressBookWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans druid window branch, and populates structure DruidWindow.
 * PRE          :
 */
void
populateDruidWindow(xmlNode * druidWindowNode, 
                    struct DruidWindow * druidWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = druidWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/druidWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), druidWindow->title.text);
      }
    }

    populateDruidWindow(cur_node->children, druidWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans accounts window branch, and populates structure AccountsWindow.
 * PRE          :
 */
void
populateAccountsWindow(xmlNode * accountsWindowNode, 
                       struct AccountsWindow * accountsWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = accountsWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/accountsWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), accountsWindow->title.text);
      }
    }

    populateAccountsWindow(cur_node->children, accountsWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans main windoe branch, and populates structure MainWindow.
 * PRE          :
 */
void
populateMainWindow(xmlNode * mainWindowNode, 
                   struct MainWindow * mainWindow)
{
  xmlNode *cur_node = NULL;

  for (cur_node = mainWindowNode; cur_node; cur_node = cur_node->next)
  {
    if(cur_node->type == XML_ELEMENT_NODE)
    {
      if(match(cur_node, "/ekiga/mainWindow/title/text"))
      {
        cleanContent(xmlNodeGetContent(cur_node), mainWindow->title.text);
      }
    }

    populateMainWindow(cur_node->children, mainWindow);
  }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Scans xml tree, and populates structure userInterface.
 * PRE          :  node is the root node to consider, and userInterface is where
            node's values are stored.
 */
void
populateUserInterfaceStruct(xmlNode * node, 
                            struct UI * userInterface)
{
    xmlNode *cur_node = NULL;

    for (cur_node = node; cur_node; cur_node = cur_node->next)
    {
      if(cur_node->type == XML_ELEMENT_NODE)
      {
        /* main window branch */
        if(match(cur_node, "/ekiga/mainWindow"))
        {
          populateMainWindow(cur_node, &userInterface->mainWindow);
        }

        /* accounts window branch */
        else if(match(cur_node, "/ekiga/accountsWindow"))
        {
          populateAccountsWindow(cur_node, &userInterface->accountsWindow);
        }

        /* druid window branch */
        else if(match(cur_node, "/ekiga/druidWindow"))
        {
          populateDruidWindow(cur_node, &userInterface->druidWindow);
        }

        /* address book window branch */
        else if(match(cur_node, "/ekiga/addressBookWindow"))
        {
          populateAddressBookWindow(cur_node, &userInterface->addressBookWindow);
        }

        /* chat window branch */
        else if(match(cur_node, "/ekiga/chatWindow"))
        {
          populateChatWindow(cur_node, &userInterface->chatWindow);
        }

        /* history window branch */
        else if(match(cur_node, "/ekiga/historyWindow"))
        {
          populateHistoryWindow(cur_node, &userInterface->historyWindow);
        }

        /* calls history window branch */
        else if(match(cur_node, "/ekiga/callsHistoryWindow"))
        {
          populateCallsHistoryWindow(cur_node, &userInterface->callsHistoryWindow);
        }

        /* preferences window branch */
        else if(match(cur_node, "/ekiga/prefsWindow"))
        {
          populatePrefsWindow(cur_node, &userInterface->prefsWindow);
        }

        /* pc to phone window branch */
        else if(match(cur_node, "/ekiga/Pp2phoneWindow"))
        {
          populatePc2phoneWindow(cur_node, &userInterface->pc2phoneWindow);
        }
      }

      populateUserInterfaceStruct(cur_node->children, 
                                  userInterface);
    }
}


/* DESCRIPTION  :  /
 * BEHAVIOR     :  Populate structure userInterface with values found in file xmlUIConfFile
 * PRE          :  xmlUIConfFile is a xml file that contains the user interface's parameters
 *          such as names and positions. And userInterface is a structure that stores all
 *          these information.
 */
int parse(char * xmlUIConfFile,
          struct UI * userInterface)
{
    xmlDoc *doc = NULL;
    xmlNode *root_element = NULL;

    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION

    /*parse the file and get the DOM */
    doc = xmlReadFile(xmlUIConfFile, NULL, 0);

    if (doc == NULL)
    {
        printf("error: could not parse file %s\n", xmlUIConfFile);
        return -1;
    }

    /*Get the root element node */
    root_element = xmlDocGetRootElement(doc);

    populateUserInterfaceStruct(root_element, userInterface);

    /*free the document */
    xmlFreeDoc(doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();

    return 0;
}
